﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=5.9
@EndOfDesignText@
' Class Module CustomLineChart
' Creating a line chart based upon the JavaScript plugin Chart.js.
' Reference: http://www.chartjs.org/docs/latest/
' Author Robert W.B. Linn @rwblinn.de
' Version 20171202
'
' TODO:
' Add more chart properties, like line color, line tension
'
' USAGE:
' Sub Class_Globals
'	Dim custLineChart As CustomLineChart
'
' Sub BuildPage()
'	page.AddExtraJavaScriptFile("custom/chart.min.js")
'
' Sub ConnectPage()
' 	Dim LineChartProperties as Map = CreateMap("label":"label", "value":Humidity, "min":0, "max":100, "maxsamples":10 )
'	custLineChart.Initialize(page, "custLineChart", LineChartProperties)
'	page.Cell(Row,1).AddComponent(custLineChart.ABMComp)
'	
' To update the datapoints, see Sub UpdateDataPoints
'
Sub Class_Globals
	Public ABMComp As ABMCustomComponent
	Private mProperties As Map
	Private mMaxSamples As Int = 100
	Public TimeStampFirstSample As String
	Public TimeStampLastSample As String
End Sub

' Initializes the object. 
' Properties - Map holding chart properties.
Public Sub Initialize(InternalPage As ABMPage, ID As String, Properties As Map)
	ABMComp.Initialize("ABMComp", Me, InternalPage, ID, "")
	mProperties = Properties
	mMaxSamples = mProperties.Get("maxsamples")
End Sub

' Runs when an object is created for the first time. Expects a valid html string which will get surrounded by a div automatically
'The <div></div> is required to enable
'* the canvas selector else canvas.js can not find the canvas selector
'* setting the canvas style to handle responsiveness (set options options: maintainAspectRatio: False, responsive: True
'* the style canvas border is not used but to show that also for the canvas tag, style settings can be used
'IMPORTANT:
'Each var holding dedicated value, array of values & array of labels must be unique to the customcomponent, therefor the internalID is used as prefix!
Sub ABMComp_Build(InternalPage As ABMPage, internalID As String) As String
	Dim html As String = $"<!-- Pre HTML definition-->
			<style>
				canvas {
					border: 0px dotted red;
				}
				chart-container {
  					position: relative;
					margin: auto;
					height: 80vh;
					width: 80vw;
				}
			</style>
			<div id="${internalID}chart" class="chart-container">
				<canvas id="${internalID}canvas"></canvas>
			</div>
			<script>
			 	var _${internalID};
				var samples = ${mMaxSamples};
				var speed = 250;
				var ${internalID}values = [];
				var ${internalID}labels = [];
				var ${internalID}value = 0;
			</script>
			"$
	Return html
End Sub

' Create a new chart by invoking JS with settings.
' The chart line & shadow is lightblue, responsive is true (requires CSS settings done in abmcomp_build).
Sub ABMComp_FirstRun(InternalPage As ABMPage, internalID As String)
	Dim script As String = $"//Line chart init with length = 0
		${internalID}values.length = 0;
		${internalID}labels.length = 0;

		var ctx = document.getElementById('${internalID}canvas');
		_${internalID} = new Chart(ctx, {
    			type: 'line',
			    data: {
      				labels: ${internalID}labels,
					datasets: [{
						label: '${mProperties.Get("label")}',
        				data: ${internalID}values,
        				// Color lightred = 255, 99, 132
        				backgroundColor: 'rgba(0, 0, 139, 0.1)',
				        borderColor: 'rgb(0, 0, 139)',
				        borderWidth: 2,
				        lineTension: 0.25,
				        pointRadius: 0.5
				     }]
    			},
			    options: {
					maintainAspectRatio: false,
      				responsive: true,
					animation: {
						duration: speed * 1.5,
        				easing: 'linear'
      				},
      				legend: false,
      				scales: {
        				xAxes: [{
          					display: false
					    }],
        				yAxes: [{
          					ticks: {
					            max: ${mProperties.Get("max")},
					            min: ${mProperties.Get("min")}
          					}
        				}]
			      	}
    			}
			});
	"$
'	Log(script)
	InternalPage.ws.Eval(script, Array As Object(ABMComp.ID))
	' flush not needed, it's done in the refresh method in the lib
End Sub

' runs when a refresh is called
Sub ABMComp_Refresh(InternalPage As ABMPage, internalID As String)
'	Dim script as string = $""$
'	InternalPage.ws.Eval(script, Array As Object(ABMComp.ID))
End Sub

' do the stuff needed when the object is removed
Sub ABMComp_CleanUp(InternalPage As ABMPage, internalID As String)
'	
End Sub

#Region DataPoints
' Update all data points
' Data - String in JSON format holding an array of maps with the keys timestamp (in "") and value.
' Example [ { "timestamp":"20:50", "value":55.4 },{ "timestamp":"20:50", "value":55.6 } ]
' The data is assigned to a JS array which length is reset to 0. The number of datapoints is determined by the length of the map.
Sub UpdateDataPoints(InternalPage As ABMPage, internalID As String, Data As String)
	Dim script As String = $"
		${internalID}values.length = 0;
		${internalID}labels.length = 0;
		var data = ${Data};
		for(var i in data) {
				// console.log(data[i].value);
				${internalID}values.push(data[i].value);
				${internalID}labels.push(data[i].timestamp);
		}
		_${internalID}.update();
		"$	
	InternalPage.ws.Eval(script, Null)
End Sub

'NOT USED
' Add a new datapoint label=timestamp, value=brickletvalue. Update the first and last timestamp.
' This requires to init the array in the sub firstrun like below
'		${internalID}values.length = samples;
'		${internalID}labels.length = samples;
'		${internalID}values.fill(Null);
'		${internalID}labels.fill(Null);
' The JS returns an array holding the first & last timestamp.
Sub AddDataPoint(InternalPage As ABMPage, internalID As String, Label As String, Value As Double)	'ignore
	Dim script As String = $"//Add data point
								${internalID}values.push(${Value});
								${internalID}values.shift();
								${internalID}labels.push('${Label}')
								${internalID}labels.shift();
								_${internalID}.update();
								// Keep track of the first and last timestamp
								var ids = [];
								var first = ${internalID}labels[${internalID}labels.lastIndexOf(null) + 1];
								var last  = ${internalID}labels[${internalID}labels.length-1];
								ids.push(first);
								ids.push(last);
								return ids;
						   "$
'	Log(script)
	' Run the script and handle the result, which is an array with 2 values first & last time stamp.
	Dim f As Future = InternalPage.ws.EvalWithResult(script, Null)
	' Convert the future to a list and assign the 2 values to globals.
	Dim l As List = f.Value
	TimeStampFirstSample = l.Get(0)
	TimeStampLastSample = l.Get(1)
End Sub
#End Region
