﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=StaticCode
Version=5.9
@EndOfDesignText@
' Class Module TFDataSampler
' Handle the datacollection from the sensors via timer and update the connected clients browser page
' The data is stored in a list with maps holding sample timestamp and the list of sensors with value. All keys must be in lowercase.
' Example map entry: "timestamp":TimeStamp, "humidity":Humidity, "temperature":Temperature, "airpressure":AirPressure
' 
' Version 20171129
Private Sub Process_Globals
	Public Settings As Map
	Private SettingsFile As String = "weatherdashboard.set"

	' Measured Data
	Private Humidity As Double 
	Private Temperature As Double
	Private AirPressure As Double
	' ADD MORE 

	' TinkerForge Devices Object - for reading the data from connected bricklets
	Public TF As TFDevices
	' ADD ANOTHER Device Object - i.e.
	' Public Arduino as ArduinoSensors

	' Sample timer reads the data from the bricklets in intervals as set by the settings.
	' The timer is public to start / stop remote
	Public SampleTimer As Timer
	Public MaxSamples As Int
	Private SampleCounter As Int = 0
	Public SampleData As List
	Public SampleResult As Boolean
	Public SampleLogMessage As String
End Sub

Public Sub Initialize
	' Set time format
	DateTime.TimeFormat = "HH:mm"
	' Init the tinkerforge devices to get the data
	TF.Initialize
	' ADD MORE DEVICE SAMPLERS
	' Read the settings
	ReadSettings
	' Init the sampledata and the result vars from the device samplers, i.e. TFSampler
	MaxSamples = Settings.Get("maxsamples")
	SampleData.Initialize
	SampleResult = False
	SampleLogMessage = $"${DateTime.Time(DateTime.Now)} Collecting data ..."$
	' Init and start the sample timer
	SampleTimer.Initialize("SampleTimer", Settings.get("samplinginterval") * 1000)
	SampleTimer.Enabled = True
End Sub

' Get the data in regular intervals from the device samplers
Public Sub SampleTimer_Tick
	' Time Stamp sample taken
	Dim TimeStamp As String = DateTime.Time(DateTime.Now)
	' Data from TinkerForge devices
	TFSampler
	' ADD MORE DEVICE SAMPLER

	' Check the sample result - if true then all well else handle error
	If SampleResult = True Then
		SampleLogMessage = $"${DateTime.Time(DateTime.Now)}: Sampling status OK"$
		' Update the sample data list - check if max sample count is reached, then remove the first entry.
		SampleCounter = SampleCounter + 1
		If SampleData.Size >= MaxSamples Then
			SampleData.RemoveAt(0)
		End If
		' Create a new map holding the timestamp and sample data
		Dim DataMap As Map = CreateMap("timestamp":TimeStamp, _
									   "humidity":Humidity, "temperature":Temperature, "airpressure":AirPressure)
		' Add new sample to the sample data map holding all samples
		SampleData.Add(DataMap)	
	End If

	' Something went wrong during sampling - stop sampling
	If SampleResult = False Then
		SampleLogMessage = $"{C:#FF0000}${DateTime.Time(DateTime.Now)} Sampling stopped{/C}{BR}${SampleLogMessage}"$
		SampleTimer.Enabled = False
	End If
	
	' Update each connected browser by calling the sub updatepage from class WeatherPage.
	' The weatherpage needs to extract the required data from the SampleData to be able to display in gauges, charts etc.
	For Each clt As WeatherDashboardPage In Main.ConnectedBrowsers.Keys
		CallSubDelayed(clt, "updatepage")
	Next
End Sub

#Region Samplers
' Define the samplers - see class TFDevices for an example

' Get data from the TinkerForge devices. The measured data are stored as globals, i.e. Humidity.
' The result getting the data is kept in a flag Result & LogMessage - set in class TFDevices
Private Sub TFSampler
	' TinkerForge - get the sensor data and the status - the status is kept after reading from the last sensor.
	Dim Humidity As Double		= TF.GetHumidity(Settings.get("tfhost"), Settings.get("tfport"), Settings.get("tfhumiditybrickletuid"))
	Dim Temperature As Double	= TF.GetTemperature(Settings.get("tfhost"), Settings.get("tfport"), Settings.get("tftemperaturebrickletuid"))
	Dim AirPressure As Double	= TF.GetAirPressure(Settings.get("tfhost"), Settings.get("tfport"), Settings.get("tfbarometerbrickletuid"))
	' ADD MORE TINKERFORGE DEVICES - i.e. AmbientLight

	' Handle status of the sampling
	' TODO: for now the status is kept after reading from the last tinkerforge device. Consider to check after every reading.
	SampleResult = TF.Result
	SampleLogMessage = TF.LogMessage
End Sub

' ADD MORE SAMPLERS, i.e.
'Private Sub ArduinoSampler
' ADD
'End Sub

#End Region

#Region Settings
Public Sub ReadSettings
	If Not(File.Exists(File.DirApp, SettingsFile)) Then
		Settings = CreateMap("tfhost":"192.168.0.59", "tfport":4223, _ 
							 "tfhumiditybrickletuid":"kcz", "tftemperaturebrickletuid":"nfu", "tfbarometerbrickletuid":"jXZ", _ 
							 "samplinginterval":10, "maxsamples":100, _
							 "humiditymin": 50, "humiditymax": 100, _
							 "temperaturemin": 10, "temperaturemax": 20, _
							 "airpressuremin": 950, "airpressuremax": 1020)
		File.WriteMap(File.DirApp, SettingsFile, Settings)
	End If
	Settings = File.ReadMap(File.DirApp, SettingsFile)
End Sub

Public Sub WriteSettings
	File.WriteMap(File.DirApp, SettingsFile, Settings)
End Sub
#End Region

