[ABMaterial] Raspberry Pi Service Setup
Version 20171130
[b]Objectives[/b]
To install a service on a Raspberry Pi enabling start, stop, status and autostart an ABMaterial application.
The tutorial uses an ABMaterial example application, called WeatherDashboard, but can be applied to any other application.

The setup has been tested on a Raspberry Pi 3 with Raspian Stretch (Linux 4.9.59-v7+ arm LE).

[b]Prepare[/b]
Login as user Pi.
Create folder /home/pi/weatherdashboard
Copy the required application files, including subfolder www to /home/pi/weatherdashboard.
The folder /home/pi/weatherdashboard contains:
files: weatherdashboard.jar, weatherdashboard.set, copymewithjar.needs, robots.txt
folders: logs, www

Make changes to weatherdashboard.set accordingly.

[b]Test[/b]
After copying the required files & folder, perform a first test if the application is running, from a terminal (i.e. opened with PuTTY):
[code]
cd /home/pi/weatherdashboard
sudo java -jar /home/pi/weatherdashboard.jar
[/code]

with output example:
[code]
cd weatherdashboard
sudo java -jar weatherdashboard.jar
loading /home/pi/weatherdashboard: copymewithjar.needs...
Using cache system: 3.0
Needs material/awesome icons
Building core.min.1511971058261.css...
2017-11-29 16:57:39.683:INFO::main: Logging initialized @2700ms to org.eclipse.jetty.util.log.StdErrLog
2017-11-29 16:57:40.233:INFO:oejs.Server:main: jetty-9.4.z-SNAPSHOT
2017-11-29 16:57:40.558:INFO:oejs.session:main: DefaultSessionIdManager workerName=node0
...
[/code]

The application is running and starts to sample data.
Open webbrowser and enter as url (change the ip address): http://ip-address:51050/WeatherDashboard
The Weather Dashboard should be displayed, showing already some sampled data.

Next is to define the application as a service with start, stop & status functionality.
Note: the application path & name is in lowercase.

[b]Create Script weatherdashboard.sh[/b]
This script file is used to start the service. Important is to start the service from the weatherdashboard folder.
[code]
cd /home/pi/weatherdashboard
sudo nano weatherdashboard.sh
[/code]
with content:
[code]
#!/bin/bash
echo Starting WeatherDashboard
cd /home/pi/weatherdashboard
## Start WeatherDashboard
/usr/bin/java -jar weatherdashboard.jar
[/code]

[b]Make Script weatherdashboard.sh executable[/b]
[code]
cd /home/pi/weatherdashboard
sudo chmod +x weatherdashboard.jar
sudo chmod +x weatherdashboard.sh
[/code]

[b]Test Script weatherdashboard.sh[/b]
Run weatherdashboard.sh:
[code]
cd /home/pi/weatherdashboard
./weatherdashboard.sh
[/code]
Open the application in a webbrowser with url http://ip-address:51050/WeatherDashboard
with output example:
[code]
Starting WeatherDashboard
loading /home/pi/weatherdashboard:
...
[/code]

[b]Create Service weatherdashboard.service[/b]
[code]
sudo nano /lib/systemd/system/weatherdashboard.service
[/code]
with content:
[code]
[Unit]
Description=WeatherDashboard Service
After=network.target

[Service]
Type=simple
ExecStart=/home/pi/weatherdashboard/weatherdashboard.sh
SuccessExitStatus=143
Restart=on-abort

[Install]
WantedBy=multi-user.target
[/code]

[b]Activate weatherdashboard.service[/b]
A few more steps required, to activate the service:
[code]
sudo chmod 644 /lib/systemd/system/weatherdashboard.service
sudo systemctl enable weatherdashboard.service
sudo systemctl start weatherdashboard.service
sudo systemctl status weatherdashboard.service
[/code]
with output example:
[code]
weatherdashboard.service - WeatherDashboard Service
   Loaded: loaded (/lib/systemd/system/weatherdashboard.service; enabled; vendor preset: enabled)
   Active: active (running) since Wed 2017-11-29 19:10:35 CET; 3s ago
 Main PID: 8261 (weatherdashboar)
   CGroup: /system.slice/weatherdashboard.service
           ├─8261 /bin/bash /home/pi/weatherdashboard/weatherdashboard.sh
           └─8262 /usr/bin/java -jar weatherdashboard.jar

Nov 29 19:10:38 59 weatherdashboard.sh[8261]: 2017-11-29 19:10:38.126:INFO:oejsh.ContextHandler:main: Stopped o.e.j.
...
[/code]

[b]Summary Service Commands[/b]
[i]Check status[/i]
[code]
sudo systemctl status weatherdashboard.service
[/code]
 [i]Start service[/i]
[code]
sudo systemctl start weatherdashboard.service
[/code]
[i]Stop service[/i]
[code]
sudo systemctl stop weatherdashboard.service
[/code]
[i]Check service log[/i]
[code]
sudo journalctl -f -u weatherdashboard.service
[/code]

[b]Hints[/b]
[i]Ownership[/i]
Check if all files & folders in /home/pi/weatherdashboard have pi as the owner and not root!
If not the case, then change ownership using chown. Examples:
[code]
sudo chown pi logs
sudo chown pi weatherdashboard.sh
[/code]

[i]Reviews Regulary[/i]
Review or cleanup the weatherdasboard log files located in folder /home/pi/weatherdashboard/logs.
Check the status of the application by running [code]sudo systemctl status weatherdashboard.service[/code]

