﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=5.9
@EndOfDesignText@
' Class module SettingsPage
' Handle settings and start / stop sample timer
'
' Refs:
' https://github.com/RobinHerbots/Inputmask
Sub Class_Globals
	Private ws As WebSocket 'ignore
	' will hold our page information
	Public page As ABMPage
	' page theme
	Private theme As ABMTheme
	' to access the constants
	Private ABM As ABMaterial 'ignore	
	' name of the page, must be the same as the class name (case sensitive!)
	Public Name As String = "SettingsPage"  '<-------------------------------------------------------- IMPORTANT
	' will hold the unique browsers window id
	Private ABMPageId As String = ""
	
	' your own variables
	Private myToastId As Int
	' Input fields for the settings
	Private Input_SamplingInterval As ABMInput
	Private Input_MaxSamples As ABMInput
	' TinkerForge
	Private Input_TFHost As ABMInput
	Private Input_TFPort As ABMInput
	Private Input_TFHumidityUID As ABMInput
	Private Input_TFTemperatureUID As ABMInput
	Private Input_TFBarometerUID As ABMInput
	Private Input_HumidityMin As ABMInput
	Private Input_HumidityMax As ABMInput
	Private Input_TemperatureMin As ABMInput
	Private Input_TemperatureMax As ABMInput
	Private Input_AirPressureMin As ABMInput
	Private Input_AirPressureMax As ABMInput
End Sub

'Initializes the object. You can add parameters to this method if needed.
Public Sub Initialize
	' build the local structure IMPORTANT!
	BuildPage
End Sub

Private Sub WebSocket_Connected (WebSocket1 As WebSocket)
	Log("Connected")
	ws = WebSocket1		
	ABMPageId = ABM.GetPageID(page, Name,ws)
	Dim session As HttpSession = ABM.GetSession(ws, ABMShared.SessionMaxInactiveIntervalSeconds)
	'----------------------START MODIFICATION 4.00-------------------------------
	If session.IsNew Then
		session.Invalidate
		ABMShared.NavigateToPage(ws, "", "./")
		Return
	End If
	'----------------------END MODIFICATION 4.00-------------------------------
	
	If ABMShared.NeedsAuthorization Then
		If session.GetAttribute2("IsAuthorized", "") = "" Then
			ABMShared.NavigateToPage(ws, ABMPageId, "../")
			Return
		End If
	End If		
	
	ABM.UpdateFromCache(Me, ABMShared.CachedPages, ABMPageId, ws)
	If page.ComesFromPageCache Then
    	' when we have a page that is cached it doesn't matter if it comes or not from a new connection we serve the cached version.
		Log("Comes from cache")
    	page.Refresh
    	page.FinishedLoading
	Else
    	If page.WebsocketReconnected Then
			Log("Websocket reconnected")
        	' when we have a client that doesn't have the page in cache and it's websocket reconnected and also it's session is new - basically when the client had internet problems and it's session (and also cache) expired before he reconnected so the user has content in the browser but we don't have any on the server. So we need to reload the page.
        	' when a client that doesn't have the page in cache and it's websocket reconnected but it's session is not new - when the client had internet problems and when he reconnected it's session was valid but he had no cache for this page we need to reload the page as the user browser has content, reconnected but we have no content in cache
        	ABMShared.NavigateToPage (ws, ABMPageId, "./" & page.PageHTMLName)
    	Else
        	' when the client did not reconnected it doesn't matter if the session was new or not because this is the websockets first connection so no dynamic content in the browser ... we are going to serve the dynamic content...
        	Log("Websocket first connection")
			page.Prepare
        	ConnectPage
    	End If
	End If
	Log(ABMPageId)
End Sub

Private Sub WebSocket_Disconnected
	Log("Disconnected")
End Sub

Sub Page_ParseEvent(Params As Map)
	Dim eventName As String = Params.Get("eventname")
	Dim eventParams() As String = Regex.Split(",",Params.Get("eventparams"))
	If eventName = "beforeunload" Then
		Log("preparing for url refresh")
		ABM.RemoveMeFromCache(ABMShared.CachedPages, ABMPageId)
		Return
	End If
	Dim caller As Object = page.GetEventHandler(Me, eventName)
	If caller = Me Then
		If SubExists(Me, eventName) Then
			Params.Remove("eventname")
			Params.Remove("eventparams")
			' BEGIN NEW DRAGDROP
			If eventName = "page_dropped" Then
				page.ProcessDroppedEvent(Params)
			End If
			' END NEW DRAGDROP
			Select Case Params.Size
				Case 0
					CallSub(Me, eventName)
				Case 1
					CallSub2(Me, eventName, Params.Get(eventParams(0)))
				Case 2
					If Params.get(eventParams(0)) = "abmistable" Then
						Dim PassedTables As List = ABM.ProcessTablesFromTargetName(Params.get(eventParams(1)))
						CallSub2(Me, eventName, PassedTables)
					Else
						CallSub3(Me, eventName, Params.Get(eventParams(0)), Params.Get(eventParams(1)))
					End If
				Case Else
					' cannot be called directly, to many param
					CallSub2(Me, eventName, Params)
			End Select
		End If
	Else
		CallSubDelayed2(caller, "ParseEvent", Params) 'ignore
	End If
End Sub

public Sub BuildTheme()
	' start with the base theme defined in ABMShared
	theme.Initialize("pagetheme")
	theme.AddABMTheme(ABMShared.MyTheme)
	
	' add additional themes specific for this page
	
End Sub

public Sub BuildPage()
	' initialize the theme
	BuildTheme
	
	' initialize this page using our theme
	page.InitializeWithTheme(Name, "/ws/" & ABMShared.AppName & "/" & Name, False, ABMShared.SessionMaxInactiveIntervalSeconds, theme)
	page.ShowLoader=True
	page.PageHTMLName = "index.html"
	page.PageTitle = "Settings"
	page.PageDescription = "Settings"
	page.PageKeywords = ""
	page.PageSiteMapPriority = ""
	page.PageSiteMapFrequency = ABM.SITEMAP_FREQ_YEARLY
		
	' adding a navigation bar
	ABMShared.BuildNavigationBar(page, "Settings","", "", "", "")

	' Create the page grid

	' R1 1x12 = [Label SamplingHeader]
	page.AddRows(1,True, "").AddCells12(1,"")
	' R2 1x6 + 1x6 = [Button Start][Button Stop]
	page.AddRows(1,True, "").AddCellsOS(2,0,0,0,6,6,6,"")
	' R3 1x4 + 1x4 + 1x4 = [Input samplinginterval][Input maxsamples][Label Info]
	page.AddRows(1,True,"").AddCellsOS(1,0,0,0,4,4,4,"").AddCellsOS(1,0,0,0,4,4,4,"").AddCellsOS(1,0,0,0,4,4,4,"")

	' R4 1x12 = [Label TinkerForgeHeader]
	page.AddRows(1,True, "").AddCells12(1,"")
	' R5 1x4 + 1x4 + 1x4 = [Input tfhost][Input tfport][Label Info]
	page.AddRows(1,True,"").AddCellsOS(1,0,0,0,4,4,4,"").AddCellsOS(1,0,0,0,4,4,4,"").AddCellsOS(1,0,0,0,4,4,4,"")
	' R6 1x4 + 1x4 + 1x4 = [Input tfhumidity][Input tftemperature][Input tfbarometer]
	page.AddRows(1,True,"").AddCellsOS(1,0,0,0,4,4,4,"").AddCellsOS(1,0,0,0,4,4,4,"").AddCellsOS(1,0,0,0,4,4,4,"")
	
	' R7 1x4 + 1x4 + 1x4 = [Input tfhumidity][Input tftemperature][Input tfbarometer]
	page.AddRows(1,True,"").AddCellsOS(1,0,0,0,2,2,2,"").AddCellsOS(1,0,0,0,2,2,2,"").AddCellsOS(1,0,0,0,2,2,2,"").AddCellsOS(1,0,0,0,2,2,2,"").AddCellsOS(1,0,0,0,2,2,2,"").AddCellsOS(1,0,0,0,2,2,2,"")

	' R8 1x6 + 1x6 = [Button Save][Label Info]
	page.AddRows(1,True,"").AddCellsOS(1,0,0,0,6,6,6,"").AddCellsOS(1,0,0,0,6,6,6,"")

	'IMPORTANT once you loaded the complete grid AND before you start adding components
	page.BuildGrid 
End Sub

public Sub ConnectPage()
	Dim Row As Int
	' Connect the navigation bar
	ABMShared.ConnectNavigationBar(page)

	' Add the components
	' R1 1x12 = [Label SamplingHeader]
	Row = 1
	page.Cell(Row,1).AddComponent(ABMShared.BuildParagraph(page,"SamplingHeader","Sampling") )

	' R2 1x4 + 1x8 = [Sampling Start Button][Sampling Stop Button]
	Row = 2
	Dim Button_SamplingStart As ABMButton
	Button_SamplingStart.InitializeFlat(page, "Button_SamplingStart", "", "", "Start", "green")
	Button_SamplingStart.Enabled = Not(DataSampler.SampleTimer.Enabled)
	page.Cell(Row,1).AddComponent(Button_SamplingStart)
	Dim Button_SamplingStop As ABMButton
	Button_SamplingStop.InitializeFlat(page, "Button_SamplingStop", "", "", "Stop", "red")
	Button_SamplingStop.Enabled = DataSampler.SampleTimer.Enabled
	page.Cell(Row,1).AddComponent(Button_SamplingStop)
	Dim labelStatusInfo As ABMLabel
	Dim Status As String = $"Running: ${DataSampler.SampleTimer.Enabled}{BR}Status: ${DataSampler.SampleLogMessage}"$
	labelStatusInfo.Initialize(page, "labelStatusInfo", Status, ABM.SIZE_SMALL, False, "alignleft")
	page.Cell(Row,2).AddComponent(labelStatusInfo)

	' R3 1x4 + 1x4 + 1x4 = [Input samplinginterval][Input maxsamples][Label Info]
	Row = 3
'	Dim Input_SamplingInterval As ABMInput
	Input_SamplingInterval.Initialize(page, "Input_SamplingInterval", ABM.INPUT_NUMBER, "Sampling Interval (s)", False, "")
	Input_SamplingInterval.PlaceHolderText = "10"
'	TODO Input_SamplingInterval.inputMask = "'mask': 'd/m/y'"

'	Input_SamplingInterval.InputMask = 
	page.Cell(Row,1).AddComponent(Input_SamplingInterval)
'	Dim Input_MaxSamples As ABMInput
	Input_MaxSamples.Initialize(page, "Input_MaxSamples", ABM.INPUT_NUMBER, "Max Samples", False, "")
	Input_MaxSamples.PlaceHolderText = "720"
	page.Cell(Row,2).AddComponent(Input_MaxSamples)
	Dim labelSampling As ABMLabel
	labelSampling.Initialize(page, "labelSampling", "Interval & Max samples must be greater 0. ", ABM.SIZE_SMALL, False, "bluegrey")
	page.Cell(Row,3).AddComponent(labelSampling)

	' R4 1x12 = [Label TinkerForgeHeader]
	Row = 4
	page.Cell(Row,1).AddComponent(ABMShared.BuildParagraph(page,"TinkerForgeHeader","TinkerForge") )

	' R5 1x4 + 1x4 + 1x4 = [Input tfhost][Input tfport][Label Info]
	Row = 5
'	Dim Input_TFHost As ABMInput
	Input_TFHost.Initialize(page, "Input_TFHost", ABM.INPUT_TEXT, "Host", False, "")
	Input_TFHost.PlaceHolderText = "localhost"
	page.Cell(Row,1).AddComponent(Input_TFHost)
'	Dim Input_TFPort As ABMInput
	Input_TFPort.Initialize(page, "Input_TFPort", ABM.INPUT_NUMBER, "Port", False, "")
	Input_TFPort.PlaceHolderText = "4223"
	page.Cell(Row,2).AddComponent(Input_TFPort)
	Dim labelTFHostPort As ABMLabel
	labelTFHostPort.Initialize(page, "labelTFHostPort", "Default port is 4223.", ABM.SIZE_SMALL, False, "bluegrey")
	page.Cell(Row,3).AddComponent(labelTFHostPort)

	' R6 1x4 + 1x4 + 1x4 = [Input tfhumidity][Input tftemperature][Input tfbarometer]
	Row = 6
'	Dim Input_TFHumidityUID As ABMInput
	Input_TFHumidityUID.Initialize(page, "Input_TFHumidityUID", ABM.INPUT_TEXT, "Humidity UID", False, "")
	Input_TFHumidityUID.PlaceHolderText = "UID"
	page.Cell(Row,1).AddComponent(Input_TFHumidityUID)
'	Dim Input_TFTemperatureUID As ABMInput
	Input_TFTemperatureUID.Initialize(page, "Input_TFTemperatureUID", ABM.INPUT_TEXT, "Temperature UID", False, "")
	Input_TFTemperatureUID.PlaceHolderText = "UID"
	page.Cell(Row,2).AddComponent(Input_TFTemperatureUID)
'	Dim Input_TFBarometerUID As ABMInput
	Input_TFBarometerUID.Initialize(page, "Input_TFBarometerUID", ABM.INPUT_TEXT, "Barometer UID", False, "")
	Input_TFBarometerUID.PlaceHolderText = "UID"
	page.Cell(Row,3).AddComponent(Input_TFBarometerUID)

	' R7 1x2 + 1x2 + 1x2 + 1x2 + 1x2 + 1x2 = [Input humiditymin][Input humiditymax][Input temperaturemin][Input temperaturemax][Input airpressuremin][Input airpressuremax]
	Row = 7
'	Dim Input_HumidityMin As ABMInput
	Input_HumidityMin.Initialize(page, "Input_HumidityMin", ABM.INPUT_TEXT, "Humidity Min", False, "")
	Input_HumidityMin.PlaceHolderText = "0"
	page.Cell(Row,1).AddComponent(Input_HumidityMin)
'	Dim Input_HumidityMax As ABMInput
	Input_HumidityMax.Initialize(page, "Input_HumidityMax", ABM.INPUT_TEXT, "Humidity Max", False, "")
	Input_HumidityMax.PlaceHolderText = "100"
	page.Cell(Row,2).AddComponent(Input_HumidityMax)
'	Dim Input_TemperatureMin As ABMInput
	Input_TemperatureMin.Initialize(page, "Input_TemperatureMin", ABM.INPUT_TEXT, "Temperature Min", False, "")
	Input_TemperatureMin.PlaceHolderText = "0"
	page.Cell(Row,3).AddComponent(Input_TemperatureMin)
'	Dim Input_TemperatureMax As ABMInput
	Input_TemperatureMax.Initialize(page, "Input_TemperatureMax", ABM.INPUT_TEXT, "Temperature Max", False, "")
	Input_TemperatureMax.PlaceHolderText = "0"
	page.Cell(Row,4).AddComponent(Input_TemperatureMax)
'	Dim Input_AirPressureMin As ABMInput
	Input_AirPressureMin.Initialize(page, "Input_AirPressureMin", ABM.INPUT_TEXT, "AirPressure Min", False, "")
	Input_AirPressureMin.PlaceHolderText = "900"
	page.Cell(Row,5).AddComponent(Input_AirPressureMin)
'	Dim Input_AirPressureMax As ABMInput
	Input_AirPressureMax.Initialize(page, "Input_AirPressureMax", ABM.INPUT_TEXT, "AirPressure Max", False, "")
	Input_AirPressureMax.PlaceHolderText = "1100"
	page.Cell(Row,6).AddComponent(Input_AirPressureMax)

	' R8 1x6 + 1x6 = [Button Save][Label Info]
	Row = 8
	Dim Button_Save As ABMButton
	Button_Save.InitializeFlat(page, "Button_Save", "", "", "Save", "bluegrey")
	Button_Save.Enabled = Not(DataSampler.SampleTimer.Enabled)
	page.Cell(Row,1).AddComponent(Button_Save)
	Dim labelSaveInfo As ABMLabel
	labelSaveInfo.Initialize(page, "labelSaveInfo", "Restart the application to apply updated settings.", ABM.SIZE_SMALL, False, "alignleft")
	page.Cell(Row,2).AddComponent(labelSaveInfo)

	' UPDATE INPUT FIELDS FROM SETTINGS
	Input_SamplingInterval.Text = DataSampler.Settings.Get("samplinginterval")
	Input_MaxSamples.Text = DataSampler.Settings.Get("maxsamples")
	Input_TFHost.Text = DataSampler.Settings.Get("tfhost")
	Input_TFPort.Text = DataSampler.Settings.Get("tfport")
	Input_TFHumidityUID.Text = DataSampler.Settings.Get("tfhumiditybrickletuid")
	Input_TFTemperatureUID.Text = DataSampler.Settings.Get("tftemperaturebrickletuid")
	Input_TFBarometerUID.Text = DataSampler.Settings.Get("tfbarometerbrickletuid")
	Input_HumidityMin.Text = DataSampler.Settings.Get("humiditymin")
	Input_HumidityMax.Text = DataSampler.Settings.Get("humiditymax")
	Input_TemperatureMin.Text = DataSampler.Settings.Get("temperaturemin")
	Input_TemperatureMax.Text = DataSampler.Settings.Get("temperaturemax")
	Input_AirPressureMin.Text = DataSampler.Settings.Get("airpressuremin")
	Input_AirPressureMax.Text = DataSampler.Settings.Get("airpressuremax")
	
	' IMPORTANT
	page.Refresh
	
	' NEW, because we use ShowLoaderType=ABM.LOADER_TYPE_MANUAL
	page.FinishedLoading 'IMPORTANT
	
	page.RestoreNavigationBarPosition
End Sub

' clicked on the navigation bar
Sub Page_NavigationbarClicked(Action As String, Value As String)
	page.SaveNavigationBarPosition
	ABMShared.NavigateToPage(ws, ABMPageId, Value)
End Sub

#Region BUTTONS
Sub Button_SamplingStart_Clicked(Target As String)
	DataSampler.SampleTimer.Enabled = True
	DataSampler.SampleLogMessage = $"Sampling started at ${DateTime.Time(DateTime.Now)}"$
	myToastId = myToastId + 1
	page.ShowToast("toast" & myToastId, "toastorange", DataSampler.SampleLogMessage, 5000, False)
	ABMShared.NavigateToPage(ws, ABMPageId, "../WeatherDashboardPage/index.html")
End Sub

Sub Button_SamplingStop_Clicked(Target As String)
	DataSampler.SampleTimer.Enabled = False
	DataSampler.SampleLogMessage= $"Sampling stopped at ${DateTime.Time(DateTime.Now)}"$
	myToastId = myToastId + 1
	page.ShowToast("toast" & myToastId, "toastorange", DataSampler.SampleLogMessage, 5000, False)
	ABMShared.NavigateToPage(ws, ABMPageId, "../WeatherDashboardPage/index.html")
End Sub

' Save the settings to the settingsfile holding the settingsmap - all key in lowercase!
Sub Button_Save_Clicked(Target As String)
	DataSampler.Settings.Put("samplinginterval", Input_SamplingInterval.Text)
	DataSampler.Settings.Put("maxsamples", Input_MaxSamples.Text)
	DataSampler.Settings.Put("tfhost",Input_TFHost.Text)
	DataSampler.Settings.Put("tfport",Input_TFPort.Text)
	DataSampler.Settings.Put("tfhumiditybrickletuid",Input_TFHumidityUID.Text)
	DataSampler.Settings.Put("tftemperaturebrickletuid",Input_TFTemperatureUID.Text)
	DataSampler.Settings.Put("tfbarometerbrickletuid",Input_TFBarometerUID.Text)
	DataSampler.Settings.Put("humiditymin": Input_HumidityMin.Text)
	DataSampler.Settings.Put("humiditymax": Input_HumidityMax.Text)
	DataSampler.Settings.Put("temperaturemin": Input_TemperatureMin.Text)
	DataSampler.Settings.Put("temperaturemax": Input_TemperatureMax.Text)
	DataSampler.Settings.Put("airpressuremin": Input_AirPressureMin.Text)
	DataSampler.Settings.Put("airpressuremax": Input_AirPressureMax.Text)
	'
	DataSampler.WriteSettings
	myToastId = myToastId + 1
	page.ShowToast("toast" & myToastId, "toastorange", $"Settings updated."$, 5000, False)
End Sub

#End Region
