﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=5.9
@EndOfDesignText@
' Class module TFDevices
' This class is used by the DataSampler to get data from the bricklets.
' Ensure to download (https://www.tinkerforge.com/en/doc/Downloads.html) the latest TF Java Bindings and copy to the B4J additional libraries folder.
' Each method has its own connection parameter to handle multiple installations (=different IP addresses for master brick hosts like RasPi or WiFi brick extension).
' Getters are used to obtain the data from a bricklet.
' A getter:
' * returns a measured value
' * sets the global var Result status to true (=OK) or false (=Error)
' * sets the logmessage. The logmessage is keeping the lastexception message
' Version 20171129
Sub Class_Globals
	Private joTF As JavaObject = Me
	Public Result As Boolean
	Public LogMessage As String
End Sub

'Initializes the object.
Public Sub Initialize
	Result = False
	LogMessage = ""
End Sub

#Region SingleGetters
'Get the relative humidity in %RH
'Host - TF Host address
'Port - TF Port (use default 4223 websocket)
'UID - Unique bricklet UID (get using the Brick Viewer)
'Returns the Humidity in %RH
'Example <code>
'Dim h as Double = GetHumidity("192.168.0.59", 4223, "kcz")
'</code>
Public Sub GetHumidity(Host As String, Port As Int, UID As String) As Double
	'Result from the Inline Java function TFGetHumidity
	Try
		Result = True
		Return joTF.RunMethod("TFGetHumidity", Array(Host, Port, UID))		
	Catch
		Result = False
		LogMessage = $"[ERROR] Get Humidity - ${LastException.Message}"$
		Return 9999
	End Try
End Sub

'Get the temperature in degress celcius
'Host - TF Host address
'Port - TF Port (use default 4223 websocket)
'UID - Unique bricklet UID (get using the Brick Viewer)
'Returns the Temperature in *C
'Example <code>
'Dim t as int = GetTemperature("192.168.0.59", 4223, "kcz")
'</code>
Public Sub GetTemperature(Host As String, Port As Int, UID As String) As Double
	'Result from the Inline Java function TFGetTemperature
	Try
		Result = True
		Return joTF.RunMethod("TFGetTemperature", Array(Host, Port, UID))
	Catch
		Result = False
		LogMessage = $"[ERROR] Get Temperature - ${LastException.Message}"$
		Return 9999
	End Try
End Sub

'Get the airpressure in mbar
'Host - TF Host address
'Port - TF Port (use default 4223 websocket)
'UID - Unique bricklet UID (get using the Brick Viewer)
'Returns the AirPressure in mBar
'Example <code>
'Dim ap as int = GetAirPressure("192.168.0.59", 4223, "kcz")
'</code>
Public Sub GetAirPressure(Host As String, Port As Int, UID As String) As Double
	'Result from the Inline Java function TFGetTemperature
	Try
		Result = True
		Return joTF.RunMethod("TFGetAirPressure", Array(Host, Port, UID))
	Catch
		Result = False
		LogMessage = $"[ERROR] Get AirPressure - ${LastException.Message}"$
		Return 9999
	End Try
End Sub
#End Region

#Region MultiGetters

#End Region

'Define the TinkerForge bricklet getters which are returning their values.
'Note: as adviced by tinkerforge, getters are used instead of callbacks! this means connect, get data, disconnect
'German Note related: Wir haben absichtlich alle Programme (Brick Viewer, Logger etc.) auf Getter umgestellt, damit es keine Seiteneffekte zu anderen Nutzerprogrammen gibt (keine Umkonfigurierung der Callbacks, kein Deaktivieren der Callbacks)
'References:
'https://www.tinkerforge.com/en/doc/Software/Bricklets/Humidity_Bricklet_Java.html#humidity-bricklet-java-examples
#If JAVA
// Imports - these are defined in the Tinkerforhge.jar located in the B4J additional libraries folder
import com.tinkerforge.IPConnection;
import com.tinkerforge.BrickletHumidity;
import com.tinkerforge.BrickletTemperature;
import com.tinkerforge.BrickletBarometer;

// SINGLEGETTERS

// Bricklet Humidity 1
public double TFGetHumidity(String HOST, int PORT, String UID) throws Exception {
	IPConnection ipcon = new IPConnection();
    BrickletHumidity h = new BrickletHumidity(UID, ipcon);
    ipcon.connect(HOST, PORT); 
	int humidity = h.getHumidity();
	// System.out.println("Humidity: " + humidity/10.0 + " %RH");
	ipcon.disconnect();
	return humidity/10.0;
}

// Bricklet Temperature
public double TFGetTemperature(String HOST, int PORT, String UID) throws Exception {
	IPConnection ipcon = new IPConnection();
    BrickletTemperature t = new BrickletTemperature(UID, ipcon);
    ipcon.connect(HOST, PORT); 
	short temperature = t.getTemperature();
	// System.out.println("Temperature: " + temperature/100.0 + " *C");
	ipcon.disconnect();
	return temperature/100.0;
}

// Bricklet AirPressure
public double TFGetAirPressure(String HOST, int PORT, String UID) throws Exception {
	IPConnection ipcon = new IPConnection();
    BrickletBarometer b = new BrickletBarometer(UID, ipcon);
    ipcon.connect(HOST, PORT); 
	// Get current air pressure (unit is mbar/1000)
    int airpressure = b.getAirPressure();
    // System.out.println("Air Pressure: " + airpressure/1000.0 + " mbar");
	ipcon.disconnect();
	return airpressure/1000.0;
}

// MULTIGETTERS

#End If
